library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.const.all;


entity ex_alu is
port (
	opcode: in std_logic_vector(3 downto 0);
	op1, op2: in dat_type;
	result: out dat_type;
	branch_cond: in std_logic_vector(1 downto 0);
	take_branch, excep: out std_logic
);
end entity ex_alu;

architecture behav of ex_alu is
	signal sub, sum, bor, band, shr, shl, pass: std_logic;
	signal bxor, bnot: std_logic;
	signal ex_op1, ex_op2: unsigned(dat_w-1 downto 0);
	signal or_s, and_s, xor_s: unsigned(dat_w-1 downto 0);
	signal shr_s, shl_s: unsigned(dat_w-1 downto 0);
	signal not_s: unsigned(dat_w-1 downto 0);
	signal add_s: unsigned(dat_w-1 downto 0);
	signal pass_s: unsigned(dat_w-1 downto 0);
	signal sum_op1, sum_op2, sum_res, cin: unsigned(dat_w downto 0);
	signal c, z: std_logic;
	signal alu_res: unsigned(dat_w-1 downto 0);
begin
	DECODER: process(opcode)
	begin
		bxor <= '0';
		bnot <= '0';
		sub <= '0';
		sum <= '0';
		bor <= '0';
		band <= '0';
		excep <= '0';
		shl <= '0';
		shr <= '0';
		pass <= '0';
		case opcode is
			when "0000" => sum <= '1';
			when "0001" => sub <= '1';
			when "0010" => bor <= '1';
			when "0011" => band <= '1';
			when "0100" => shr <= '1';
			when "0101" => shl <= '1';
			when "0110" => bxor <= '1';
			when "0111" => bnot <= '1';
			when "1000" => pass <= '1';
			when "1100" => pass <= '1';
			when others => excep <= '1';
		end case;
	end process;

	ex_op2 <= unsigned(not op2) when sub = '1' else unsigned(op2);
	ex_op1 <= unsigned(op1);

	sum_op1 <= '0' & ex_op1;
	sum_op2 <= '0' & ex_op2;
	cin <= (0 => sub, others => '0');
	sum_res <= sum_op1 + sum_op2 + cin;

	add_s <= sum_res(dat_w-1 downto 0) when sum = '1' or sub = '1' else
	         (others => '0');
	or_s <= ex_op1 or ex_op2 when bor = '1' else
	        (others => '0');
	xor_s <= ex_op1 xor ex_op2 when bxor = '1' else
	         (others => '0');
	not_s <= not ex_op1 when bnot = '1' else
	         (others => '0');
	and_s <= ex_op1 and ex_op2 when band = '1' else
	         (others => '0');
	shr_s <= ex_op1 srl to_integer(ex_op2) when shr = '1' else
	         (others => '0');
	shl_s <= ex_op1 sll to_integer(ex_op2) when shl = '1' else
	         (others => '0');
	pass_s <= ex_op1 when pass = '1' else
			 (others => '0');

	alu_res <= add_s or or_s or xor_s or and_s or not_s or shr_s or shl_s or pass_s;

	z <= '1' when add_s(dat_w-1 downto 0) = 0 else '0';
	c <= sum_res(dat_w);

	take_branch <= '1' when (z = '1' and branch_cond = "00") or
	                        (z = '0' and branch_cond = "01") or
	                        (c = '1' and branch_cond = "10") or
	                        (c = '0' and branch_cond = "11") else
	               '0';
	result <= std_logic_vector(alu_res);
end architecture behav;
