library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_textio.all;
use std.textio.all;

use work.const.all;

entity memory is
	port(
		clk, reset, cs: in std_logic;
		i_rd, i_wr:  in std_logic;
		i_adr: in adr_type;
		i_dat: in dat_type;
		o_dat: out dat_type 
	);
end entity memory;

architecture behav of memory is
	type mem_type is array (0 to 2**10-1) of dat_type;
	signal mem: mem_type;
begin
	process is
		variable initialised : boolean := false;
		file in_file: text open read_mode is "rom.bin";
		variable inline: line;
		variable fdat: dat_type;
		variable fadr: adr_type;
		variable adr: std_logic_vector(adr_w-3 downto 0);
	begin
		if (not initialised) then
			for i in 0 to 2**10-1 loop
				mem(i) <= X"00000000";
			end loop;

			while not endfile(in_file) loop
				readline(in_file, inline);
				hread(inline, fadr);
				read(inline, fdat);
				adr := fadr(adr_w-1 downto 2);

				mem(conv_integer(adr)) <= fdat;
			end loop;

			initialised := true;
		end if;

		wait for 60 ps;
		adr := i_adr(adr_w-1 downto 2);

		if (i_wr = '1' and i_wr'event) then
			mem(conv_integer(adr)) <= i_dat;
		end if;
		o_dat <= mem(conv_integer(adr));
		wait on i_adr, i_wr, i_rd, i_dat;
	end process;

end architecture behav;

architecture architec of memory is
        signal wr_ram, wr_rom: std_logic;
        signal ramadr: ram_adr_type;
        signal romadr: rom_adr_type;
        signal ramout, romout: dat_type;
begin
        RAM: entity work.ram(rtl)
                PORT MAP(clk, wr_ram,
                         ramadr, i_dat, ramout);

        ROM: entity work.rom(rtl)
                PORT MAP(clk, wr_rom,
                         romadr, i_dat, romout);

        DEC: entity work.memdec(rtl)
                PORT MAP(cs, i_wr, i_rd, i_adr, o_dat,
                         wr_ram, ramadr, ramout,
                         wr_rom, romadr, romout);
end architecture architec;
