library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use work.const.all;

-- this is centralized control for stalls and nops
-- inputs can be added as needed
-- outputs are fixed, only their equations should be changed

-- if one stage is stalled, all stages before it are also stalled
entity stall_nop_control is
port (
	clk, reset: in std_logic;
	id_load_alu_stall: in std_logic;
	ex_branch_misspredict: in std_logic;
	imiss, dmiss, bfull: in std_logic;
	wb_isExc: in std_logic; --NOP all stage latches
	wb_isHalt: in std_logic;
	o_stall_if: out std_logic;
	o_stall_id: out std_logic;
	o_stall_ex: out std_logic;
	o_stall_mem: out std_logic;
	o_stall_wb: out std_logic;
	o_nop_if: out std_logic;
	o_nop_id: out std_logic;
	o_nop_ex: out std_logic;
	o_nop_mem: out std_logic;
	is_halted: out std_logic
);
end entity stall_nop_control;

architecture behav of stall_nop_control is
	signal branch_miss, stall, alu_stall: std_logic;
	signal halt: std_logic;
begin
	stall <= imiss or dmiss or bfull;
	alu_stall <= id_load_alu_stall and not wb_isExc;
	branch_miss <= ex_branch_misspredict and not wb_isExc;

	--stall stage which requires stall and all stages before it
	o_stall_if <= alu_stall or stall or halt;
	o_stall_id <= stall or halt;
	o_stall_ex <= stall or halt;
	o_stall_mem <= stall or halt;
	o_stall_wb <= stall or halt; --only for perfCounters

	o_nop_if <= '0';
	o_nop_id <= alu_stall or branch_miss or wb_isExc;
	o_nop_ex <= wb_isExc;
	o_nop_mem <= wb_isExc;

	is_halted <= halt;
	haltUpdate: process(clk, reset, wb_isHalt)
	begin
		if reset = '1' then
			halt <= '0';
		elsif clk = '1' and clk'event and wb_isHalt = '1' then
			halt <= '1';
		end if;
	end process haltUpdate;
end architecture behav;
