library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

use work.const.all;


entity wb_stage is
port (
	clk: std_logic;
	reset: std_logic;
	stall: std_logic;
	ishalted: std_logic;

	pc: in adr_type;
	i_branch_cond: std_logic_vector(1 downto 0);
	--instruction type
	isValid: in std_logic;
	isMem: in std_logic;
	isLdi: in std_logic;
	isAlu: in std_logic;
	isAluSpec: in std_logic;
	isStore: in std_logic;
	isHalt: in std_logic;
	--
	rdAdr: in reg_adr_type;

	aluOut: in dat_type;
	memOut: in dat_type;
	memRegWrEna: in std_logic;

	i_exc_vector: in exc_vector_type;
	i_rmAddress: in adr_type;

	--
	wbOutDat: out dat_type;
	wbOutAdr: out reg_adr_type;
	wbEna: out std_logic;
	wb_trapPC_offset: out adr_type;
	wb_rmPC, wb_rmAddress: out adr_type;
	wb_isExc: out std_logic;
	wb_trapPC_ena: out std_logic;
	wb_ishalt: out std_logic
);
end entity wb_stage;

architecture behav of wb_stage is
	signal trapPCOffset: adr_type;
	signal activeCycles, totalCycles: perf_counter_type;
begin
	wbOutDat <= memOut when isMem = '1' else aluOut;
	wbOutAdr <= rdAdr;
	--wbEna <= isAlu or (isMem and not isStore) or isLdi;
	wbEna <= memRegWrEna;

	wb_isExc <= '1' when i_exc_vector /= "000000000" and isValid = '1' else
	            '0';
	wb_rmPC <= pc;
	wb_rmAddress <= i_rmAddress;

	trapPcOff: process(i_exc_vector)
	begin
		trapPCOffset <= (others => '0');
		for i in exc_cnt-1 downto 0 loop
			if i_exc_vector(i) = '1' then
				trapPCOffset <= std_logic_vector(to_unsigned(4*i, adr_w));
			end if;
		end loop;
	end process trapPcOff;

	perfCounters: process(clk, reset, stall, isValid)
	begin
		if reset = '1' then
			activeCycles <= (others => '0');
			totalCycles <= (others => '0');
		elsif clk = '1' and clk'event and ishalted = '0' then
			if isValid = '1' and stall = '0' then
				activeCycles <= activeCycles + 1;
			end if;
			totalCycles <= totalCycles + 1;
		end if;
	end process perfCounters;

	wb_trapPC_offset <= trapPCOffset;
	wb_trapPC_ena <= isAluSpec and i_branch_cond(0);

	wb_ishalt <= isHalt;
end architecture behav;
